function nefcrea(num, gen, contr)   

%NEFCREA Create new nefcon simulink blocks and Signal-Generators  
%   num = number of inputs  
%   gen = selected generator (1 Nefcon, 2 bounded)  
%   contr = selected controller (1 nefcon, 2 fuzzy)  
%  
% See also NEFGUI and NEFCON.  

% Author: Andreas Nuernberger, TU Braunschweig, August 1996; a.nuernberger@tu-bs.de
%         contact: Ludger Merz, DASA-Airbus GmbH, Hamburg; ludger.merz@airbus.de

% get actual name   
sys = get_param(0,'CurrentSystem');   

%if isempty(get_param([sys,'/','Signal'], 'position'))
if isempty(find_system(sys,'SearchDepth',1,'Name','Signal'))
   add_block('built-in/To Workspace',[sys,'/','Signal'])
   set_param([sys,'/','Signal'],...
      'BackGround',2,...
      'mat-name','nef_u',...
      'buffer','10000',...
      'Mask Display','nef_u',...
      'position',[300,30,360,10])
end

%if isempty(get_param([sys,'/','Response'], 'position'))   
if isempty(find_system(sys,'SearchDepth',1,'Name','Response'))
   add_block('built-in/To Workspace',[sys,'/','Response'])
   set_param([sys,'/','Response'],...
      'BackGround',2,...
      'mat-name','nef_y',...
      'buffer','10000',...
      'Mask Display','nef_y',...
      'position',[300,80,360,60])
end

%     Subsystem   
subname = 'NefconFuzzyController';   

% find existing systems   
nof = 0;   
%if ~isempty(get_param([sys,'/',subname], 'position'))  
if ~isempty(find_system(sys,'SearchDepth',1,'Name',subname))
   nof = 1;   
   %	disp('NEFCON: System already exists.');
   % Generate a beep   
   %	fprintf('\a')   
else   
   
   %while ~isempty(get_param([sys,'/',subname, num2str(nof)], 'position'))
   while ~isempty(find_system(sys,'SearchDepth',1,'Name',[subname, num2str(nof)]))
      nof = nof + 1;   
   end   
   if nof > 0   
      subname = [subname, num2str(nof)];   
   end   
   
   % create new subsystem   
   y = 20;   
   %new_system([sys,'/',subname])   
   add_block('built-in/SubSystem',[sys,'/',subname])   
   
   add_block('built-in/Mux',[sys,'/',subname,'/Mux'])   
   set_param([sys,'/',subname,'/Mux'],...   
      'inputs',num2str(num),...   
      'position',[205,y+30,235,y+45*num+45])   
   
   for i=1:num,   
      add_block('built-in/Inport',[sys,'/',subname,'/in_',num2str(i)])   
      set_param([sys,'/',subname,'/in_',num2str(i)],...   
         'Port',num2str(i),...   
         'position',[35,y+i*45,55,y+20+i*45])   
      
      add_block('built-in/Gain',[sys,'/',subname, '/GainIn', num2str(i)])   
      set_param([sys,'/',subname, '/GainIn', num2str(i)],...   
         'Gain','1',...  
         'position',[80,y+i*45,100,y+20+i*45])   
      
      add_block('built-in/Saturation',[sys,'/',subname,'/Saturation',num2str(i)])   
      set_param([sys,'/',subname,'/Saturation',num2str(i)],...   
         'Lower Limit','-1',...   
         'Upper Limit','1',...   
         'position',[130,y+i*45,155,y+20+i*45])   
      
      autoline([sys,'/',subname],['in_',num2str(i), '/1'], ...   
         ['GainIn', num2str(i), '/1']);   
      autoline([sys,'/',subname],['GainIn', num2str(i), '/1'], ...   
         ['Saturation',num2str(i), '/1']);   
      autoline([sys,'/',subname],['Saturation',num2str(i), '/1'], ...   
         ['Mux/',num2str(i)]);   
   end;   
   
   % add input if bounded signal generator is used   
   %if gen == 2   
   if 0   
      add_block('built-in/Inport',[sys,'/',subname,'/in_',num2str(num+1)])   
      set_param([sys,'/',subname,'/in_',num2str(num+1)],...   
         'Port',num2str(num+1),...   
         'position',[35,y+(num+1)*45,55,y+20+(num+1)*45])   
      if contr == 1   
         set_param([sys,'/',subname,'/Mux'],...   
            'inputs',num2str(num+1));   
         autoline([sys,'/',subname],['in_',num2str(num+1), '/1'], ...   
            ['Mux/',num2str(num+1)]);   
      end   
   end       
   
   
   mid = (y+45 + y+45*num+20)/2;   
   
   
   if contr == 1      % nefcon system   
      add_block('built-in/Demux',[sys,'/',subname,'/Demux']);   
      set_param([sys,'/',subname,'/Demux'],...   
         'outputs','4',...   
         'position',[350,mid-20,385,mid+20]);   
      add_block('built-in/Gain',[sys,'/',subname, '/GainOut'])   
      set_param([sys,'/',subname, '/GainOut'],...   
         'Gain','1',...  
         'position',[410,mid-50,430,mid-30]);   
      add_block('built-in/Outport',[sys,'/',subname,'/out_1']);   
      set_param([sys,'/',subname,'/out_1'],...   
         'position',[460,mid-50,480,mid-30]);   
      add_block('built-in/Stop Simulation',[sys,'/',subname,'/Stop Simulation']);   
      set_param([sys,'/',subname,'/Stop Simulation'],...   
         'position',[450,mid+50,490,mid+70]);   
      add_block('built-in/Scope',[sys,'/',subname,'/Error']);  
      set_param([sys,'/',subname,'/Error'],...  
         'BackGround',3,...  
         'Vmax','4.000000',...  
         'Hmax','10.000000',...  
         'position',[500,mid-10,520,mid+10]);   
      %    		'Vgain','2.000000',...  
      %    		'Hgain','5.000000',...  
      add_block('built-in/Mux',[sys,'/',subname,'/Mux1'])   
      set_param([sys,'/',subname,'/Mux1'],...   
         'inputs','2',...   
         'position',[450,mid-10,480,mid+10])   
      add_block('built-in/Constant',[sys,'/',subname,'/Zero'])   
      set_param([sys,'/',subname,'/Zero'],...   
         'Value','0',...   
         'position',[410,mid,430,mid+20])   
      add_block('built-in/Goto',[sys,'/',subname,'/NoiseFlag'])   
      set_param([sys,'/',subname,'/NoiseFlag'],...   
         'GotoTag','NoiseFlag',...   
         'TagVisibility','global',...   
         'position',[450,mid+100,490,mid+130])   
      add_block('built-in/S-Function',[sys,'/',[subname,'/Nefcon ',13,'Lern-System']]);
      set_param([sys,'/',[subname,'/Nefcon ',13,'Lern-System']],...   
         'function name','snefcon',...   
         'position',[280,mid-20,335,mid+20]);   
      
      autoline([sys,'/',subname],['Mux/1'], ...   
         ['Nefcon ',13,'Lern-System/1']);   
      autoline([sys,'/',subname],['Nefcon ',13,'Lern-System/1'], ...   
         ['Demux/1']);   
      autoline([sys,'/',subname],'Demux/1', 'GainOut/1');   
      autoline([sys,'/',subname],'GainOut/1', 'out_1/1');   
      autoline([sys,'/',subname],'Demux/2', 'Mux1/1');   
      autoline([sys,'/',subname],'Zero/1', 'Mux1/2');   
      autoline([sys,'/',subname],'Mux1/1', 'Error/1');   
      autoline([sys,'/',subname],'Demux/3', 'Stop Simulation/1');   
      autoline([sys,'/',subname],'Demux/4', 'NoiseFlag/1');   
      
   else   
      add_block('built-in/Outport',[sys,'/',subname,'/out_1'])   
      set_param([sys,'/',subname,'/out_1'],...   
         'position',[400,mid-10,420,mid+10])   
      add_block('built-in/S-Function',[sys,'/',[subname,'/Fuzzy Logic',13,'Controller']])   
      set_param([sys,'/',[subname,'/Fuzzy Logic',13,'Controller']],...   
         'function name','sffis',...   
         'parameters','fismatrix',...   
         'Mask Display','plot(0, -0.3, 6, -0.3, 0, 1.2, 6, 1.2, [0 6], [0 0], [1 2 3],[0 1 0],[2 3 4],[0 1 0],[3 4 5],[0 1 0])')   
      set_param([sys,'/',[subname,'/Fuzzy Logic',13,'Controller']],...   
         'Mask Type','FIS',...   
         'Mask Dialogue','FIS|FIS Matrix:',...   
         'Mask Translate','fismatrix = @1;',...   
         'Mask Help','Fuzzy Inference System',...   
         'Mask Entries','fismat\/')   
      set_param([sys,'/',[subname,'/Fuzzy Logic',13,'Controller']],...   
         'position',[280,mid-20,335,mid+20])   
      autoline([sys,'/',subname],['Fuzzy Logic',13,'Controller/1'], ...   
         ['out_1/1']);   
      autoline([sys,'/',subname],['Mux/1'], ...   
         ['Fuzzy Logic',13,'Controller/1']);   
   end   
   
   
   %     Finished composite block subname.   
   set_param([sys,'/',subname],...   
      'Mask Display','NEFCON\nFuzzy')   
   
   set_param([sys,'/',subname],...   
      'position',[160,10,220,100],...   
      'BackGround',2,...   
      'hide name','off',...   
      'name','NefconFuzzyController')   
   
   contrName = subname;   
   
   
   %open_system([sys,'/',subname]);   
   
   drawnow   
   
end

% -------------------------------------------------------------------   
% Nefcon-SigGen   
% -------------------------------------------------------------------   
subname = 'Nefcon-SigGen';   

%if gen==1 & isempty(get_param([sys,'/',subname], 'position'))   % look if generator exists already...   
if gen==1 & isempty(find_system(sys,'SearchDepth',1,'Name',subname))
   new_system([sys,'/',subname])   
   set_param([sys,'/',subname],'Location',[575,561,1118,865])   
   
   add_block('built-in/Signal Generator',[sys,'/',[subname, '/Signal Gen.',13,'f_rad=2*pi*f_Hz']])   
   set_param([sys,'/',[subname, '/Signal Gen.',13,'f_rad=2*pi*f_Hz']],...   
      'Peak','1.000000',...   
      'Peak Range','5.000000',...   
      'Freq','0.050000',...   
      'Freq Range','0.500000',...   
      'Wave','Sqr',...   
      'Units','Rads')   
   set_param([sys,'/',[subname, '/Signal Gen.',13,'f_rad=2*pi*f_Hz']],...   
      'position',[70,59,110,81])   
   
   add_block('built-in/Constant',[sys,'/',subname, '/Constant'])   
   set_param([sys,'/',subname, '/Constant'],...   
      'position',[155,125,175,145])   
   
   add_block('built-in/Gain',[sys,'/',subname, '/Gain1'])   
   set_param([sys,'/',subname, '/Gain1'],...   
      'position',[355,119,410,151])   
   
   add_block('built-in/Sum',[sys,'/',subname, '/Sum'])   
   set_param([sys,'/',subname, '/Sum'],...   
      'inputs','+++',...   
      'position',[290,38,320,232])   
   
   add_block('built-in/Outport',[sys,'/',subname, '/out_1'])   
   set_param([sys,'/',subname, '/out_1'],...   
      'position',[440,125,460,145])   
   %-----   
   add_block('built-in/Constant',[sys,'/',subname,'/Constant1'])  
   set_param([sys,'/',subname,'/Constant1'],...  
      'Value','0',...  
      'position',[150,240,170,260])  
   
   add_block('built-in/Switch',[sys,'/',subname,'/Switch'])  
   set_param([sys,'/',subname,'/Switch'],...  
      'Threshold','0.5',...  
      'position',[200,167,225,233])  
   
   add_block('built-in/Gain',[sys,'/',subname,'/Gain'])  
   set_param([sys,'/',subname,'/Gain'],...  
      'Gain','0.4',...  
      'position',[90,164,145,196])  
   
   %   add_block('built-in/Signal Generator',[sys,'/',subname,'/Rauschen'])  
   %   set_param([sys,'/',subname,'/Rauschen'],...  
   %   		'Peak','1.000000',...  
   %   		'Peak Range','5.000000',...  
   %   		'Freq','1.000000',...  
   %   		'Freq Range','5.000000',...  
   %   		'Wave','Rnd',...  
   %   		'Units','Rads',...  
   %   		'position',[25,169,65,191])  
   
   add_block('built-in/RandomNumber',[sys,'/',subname,'/Noise'])  
   set_param([sys,'/',subname,'/Noise'],...  
      'position',[25,169,65,191])  
   
   
   %     Subsystem  subname,'/Get Signal'.  
   
   %new_system([sys,'/',subname,'/Get Signal'])  
   %set_param([sys,'/',subname,'/Get Signal'],'Location',[200,498,478,742])  
   
   %add_block('built-in/S-Function',[sys,'/',subname,'/Get Signal/S-Function1'])  
   %set_param([sys,'/',subname,'/Get Signal/S-Function1'],...  
   %   'function name','getsig',...  
   %   'parameters','width,name',...  
   %   'position',[65,48,115,72])  
   %add_block('built-in/Outport',[sys,'/',subname,'/Get Signal/out_1'])  
   %set_param([sys,'/',subname,'/Get Signal/out_1'],...  
   %   'position',[125,50,145,70])  
   %add_line([sys,'/',subname,'/Get Signal'],[120,60;120,60])  
   %set_param([sys,'/',subname,'/Get Signal'],...  
   %   'Mask Display','GETSIG',...  
   %   'Mask Type','GEt Signal',...  
   %   'Mask Dialogue','Put a signal into the MATLAB workspace.  Specify the signal width and name. | Signal Width: | Name (in quotes):')  
   %set_param([sys,'/',subname,'/Get Signal'],...  
   %   'Mask Translate','width=@1;name=@2;')  
   %set_param([sys,'/',subname,'/Get Signal'],...  
   %   'Mask Help','Get signal from the MATLAB workspace. To be used in conjunction with PUTSIG. Names must be entered as strings.  The code for this block is getsig.c')  
   %set_param([sys,'/',subname,'/Get Signal'],...  
   %   'Mask Entries','1\/''NEFCON_NOISE''\/')  
   %     Finished composite block subname,'/Get Signal'.  
   
   %set_param([sys,'/',subname,'/Get Signal'],...  
   %   'position',[35,210,95,250])  
   
   add_block('built-in/From',[sys,'/',subname,'/NoiseFlag'])  
   set_param([sys,'/',subname,'/NoiseFlag'],...  
      'GotoTag','NoiseFlag',...  
      'position',[35,210,95,250])  
   
   
   add_line([sys,'/',subname],[180,135;285,135])  
   add_line([sys,'/',subname],[70,180;85,180])  
   add_line([sys,'/',subname],[115,70;285,70])  
   add_line([sys,'/',subname],[415,135;435,135])  
   add_line([sys,'/',subname],[325,135;350,135])  
   add_line([sys,'/',subname],[230,200;285,200])  
   add_line([sys,'/',subname],[150,180;195,180])  
   add_line([sys,'/',subname],[175,250;180,250;180,220;195,220])  
   add_line([sys,'/',subname],[100,230;140,230;140,200;195,200])  
   set_param([sys,'/',subname],...  
      'Mask Display','NEFCON\nSigGen')  
   
   %-----   
   
   %     Finished composite block subname .   
   
   set_param([sys,'/',subname ],...   
      'BackGround',2,...   
      'hide name','off',...   
      'position',[60,30,130,80])   
   
end;    % end of generator   

% -------------------------------------------------------------------   
% Nefcon-Bounded-SigGen   
% -------------------------------------------------------------------   
subname = 'Bounded SigGen';   

%if gen==2 & isempty(get_param([sys,'/',subname], 'position'))   % look if generator exists already...   
if gen==2 & isempty(find_system(sys,'SearchDepth',1,'Name',subname))
   
   %     Subsystem  subname.  
   
   new_system([sys,'/',subname])  
   set_param([sys,'/',subname],'Location',[228,66,700,333])  
   %open_system([sys,'/',subname])  
   
   add_block('built-in/Switch',[sys,'/',subname,'/Switch'])  
   set_param([sys,'/',subname,'/Switch'],...  
      'Threshold','0.5',...  
      'position',[205,23,230,117])  
   
   %    add_block('built-in/Signal Generator',[sys,'/',subname,'/Rauschen'])  
   %    set_param([sys,'/',subname,'/Rauschen'],...  
   %    		'Peak','1.000000',...  
   %    		'Peak Range','5.000000',...  
   %    		'Freq','1.000000',...  
   %    		'Freq Range','5.000000',...  
   %    		'Wave','Rnd',...  
   %    		'Units','Rads',...  
   %    		'position',[35,29,75,51])  
   add_block('built-in/RandomNumber',[sys,'/',subname,'/Noise'])  
   set_param([sys,'/',subname,'/Noise'],...  
      'position',[35,29,75,51])  
   
   add_block('built-in/Gain',[sys,'/',subname,'/Gain'])  
   set_param([sys,'/',subname,'/Gain'],...  
      'Gain','0.4',...  
      'position',[100,24,155,56])  
   
   
   %     Subsystem  subname,'/Get Signal'.  
   
   %new_system([sys,'/',subname,'/Get Signal'])  
   %set_param([sys,'/',subname,'/Get Signal'],'Location',[200,498,478,742])  
   
   %add_block('built-in/Outport',[sys,'/',subname,'/Get Signal/out_1'])  
   %set_param([sys,'/',subname,'/Get Signal/out_1'],...  
   %   'position',[125,50,145,70])  
   
   %add_block('built-in/S-Function',[sys,'/',subname,'/Get Signal/S-Function1'])  
   %set_param([sys,'/',subname,'/Get Signal/S-Function1'],...  
   %   'function name','getsig',...  
   %   'parameters','width,name',...  
   %   'position',[65,48,115,72])  
   %add_line([sys,'/',subname,'/Get Signal'],[120,60;120,60])  
   %set_param([sys,'/',subname,'/Get Signal'],...  
   %   'Mask Display','GETSIG',...  
   %   'Mask Type','GEt Signal',...  
   %   'Mask Dialogue','Put a signal into the MATLAB workspace.  Specify the signal width and name. | Signal Width: | Name (in quotes):')  
   %set_param([sys,'/',subname,'/Get Signal'],...  
   %   'Mask Translate','width=@1;name=@2;')  
   %set_param([sys,'/',subname,'/Get Signal'],...  
   %   'Mask Help','Get signal from the MATLAB workspace. To be used in conjunction with PUTSIG. Names must be entered as strings.  The code for this block is getsig.c')  
   %set_param([sys,'/',subname,'/Get Signal'],...  
   %   'Mask Entries','1\/''NEFCON_NOISE''\/')  
   
   
   %     Finished composite block subname,'/Get Signal'.  
   
   %set_param([sys,'/',subname,'/Get Signal'],...  
   %   'position',[30,70,90,110])  
   
   add_block('built-in/From',[sys,'/',subname,'/NoiseFlag'])  
   set_param([sys,'/',subname,'/NoiseFlag'],...  
      'GotoTag','NoiseFlag',...  
      'position',[30,70,90,110])  
   
   add_block('built-in/Constant',[sys,'/',subname,'/Constant1'])  
   set_param([sys,'/',subname,'/Constant1'],...  
      'Value','0',...  
      'position',[140,100,160,120])  
   
   add_block('built-in/Inport',[sys,'/',subname,'/in_1'])   
   set_param([sys,'/',subname,'/in_1'],...   
      'Port', '1',...   
      'position',[15,190,35,210])  
   
   add_block('built-in/Derivative',[sys,'/',subname,'/Derivative'])  
   set_param([sys,'/',subname,'/Derivative'],...  
      'position',[70,230,100,260])  
   
   add_block('built-in/Mux',[sys,'/',subname,'/Mux'])  
   set_param([sys,'/',subname,'/Mux'],...  
      'inputs','2',...  
      'position',[120,190,150,250])  
   
   add_block('built-in/S-Function',[sys,'/',subname,'/S-function'])  
   set_param([sys,'/',subname,'/S-function'],...  
      'function name','snefsign',...  
      'position',[175,210,235,230])  
   
   add_block('built-in/Demux',[sys,'/',subname,'/Demux'])  
   set_param([sys,'/',subname,'/Demux'],...  
      'outputs','3',...  
      'position',[260,200,280,240])  
   
   add_block('built-in/Sum',[sys,'/',subname,'/Sum'])  
   set_param([sys,'/',subname,'/Sum'],...  
      'position',[310,32,335,188])  
   
   add_block('built-in/Gain',[sys,'/',subname,'/Gain1'])  
   set_param([sys,'/',subname,'/Gain1'],...  
      'position',[355,94,410,126])  
   
   add_block('built-in/Outport',[sys,'/',subname,'/out_1'])  
   set_param([sys,'/',subname,'/out_1'],...  
      'position',[425,100,445,120])  
   
   
   autoline([sys,'/',subname],'in_1/1', 'Derivative/1');   
   autoline([sys,'/',subname],'in_1/1', 'Mux/1');   
   autoline([sys,'/',subname],'Derivative/1', 'Mux/2');   
   autoline([sys,'/',subname],'Mux/1', 'S-function/1');   
   autoline([sys,'/',subname],'S-function/1', 'Demux/1');   
   autoline([sys,'/',subname],'Demux/1', 'Sum/2');   
   autoline([sys,'/',subname],'Noise/1', 'Gain/1');   
   autoline([sys,'/',subname],'Gain/1', 'Switch/1');   
   autoline([sys,'/',subname],'NoiseFlag/1', 'Switch/2');   
   autoline([sys,'/',subname],'Constant1/1', 'Switch/3');   
   autoline([sys,'/',subname],'Switch/1', 'Sum/1');   
   autoline([sys,'/',subname],'Sum/1', 'Gain1/1');   
   autoline([sys,'/',subname],'Gain1/1', 'out_1/1');   
   
   set_param([sys,'/',subname],...   
      'Mask Display','NEFCON\nBounded\nSigGen',...   
      'Mask Dialogue','eval(''nefsign'');')   
   
   %     Finished composite block subname.   
   
   set_param([sys,'/',subname],...   
      'BackGround',2,...   
      'hide name','off',...   
      'position',[10,10,80,60])   
   
   drawnow   
   
   
end;    % end of bounded generator   

drawnow   

