function S_and_P = neffilla(fig,axs)
%NEFFILLA creates the constraint bounds and does some data checking.
%
%       Modified version of FILLAXES.
%
%       S_and_P = FILLAXES(FIG) creates the upper and lower 
%       constraint bounds and patches in the constraint figure FIG.  
%       It also does some error checking on the matrices NCD_LB and 
%       NCD_UB.
%
%       The output S_and_P contains the handles of the bounds and
%       patches.
%
%       See also MAKESURF.

%	Author(s): A. Potvin, 12-1-92
%	Copyright (c) 1992-93 by The MathWorks, Inc.
%	$Revision: 1.13 $  $Date: 1993/11/02 23:34:06 $
%       Modified: Andreas Nuernberger; DASA Airbus; June 1996

%global NCD_UB NCD_LB RANGE_LIMIT
%global RANGE_LIMIT
global ncdStruct;

% Default output (in case of error) and definitions
S_and_P = zeros(1,4);
ud = get(fig,'UserData');
out = ud(1,2);
sys = get(ud(1,6),'UserData');

% Get tstart and tfinal from SIMULINK

tstart = get_param(sys,'Start time');
if (isstr(tstart)), tstart = eval(tstart); end
tfinal = get_param(sys, 'Stop time');
if (isstr(tfinal)), tfinal = eval(tfinal); end

% Get start and final time
%if isempty(RANGE_LIMIT),
   % Get tstart and tfinal from SIMULINK
   % Use eval('try','catch') in case system name changed
   %eval([sys '([],[],[],0);'],'errorncd(10); return');
%   tstart = get_param(sys,'Start time');
%   tfinal = get_param(sys,'Stop time');
   % Hack: 4.1x get_param returns strings, 4.0 and 4.1 sometimes returned numbers
%   if isstr(tstart),
      % No need for protected eval('try','catch')
%      tstart = eval(tstart);
%      tfinal = eval(tfinal);
%   end
   
%   RANGE_LIMIT = [tstart tfinal];
%else
%   tstart = RANGE_LIMIT(1,1);
%   tfinal = RANGE_LIMIT(1,2);
%end

% If no lower bounds defined, use default
DoDefault = 0;
if isempty(ncdStruct.CnstrLB),
   DoDefault = 1;
elseif isempty(find(ncdStruct.CnstrLB(1,:)==out)),
   DoDefault = 1;
end
if DoDefault,
   dt = tfinal-tstart;
   ylim = get(gca, 'Ylim');
   %   lb = ylim(1) + (ylim(2)-ylim(1))*.1;
   %   NCD_LB = [NCD_LB, ...
   %             [out*ones(1,4); ...
   %              tstart+[0 0.5*dt 0.5*dt] tfinal; ...
   %              lb lb lb lb; ...
   %              1  0  1  0]];
   %   lb = ylim(2)-ylim(1)*0.9;
   %   NCD_LB = [NCD_LB, ...
   %             [                                out*ones(1,6); ...
   %              tstart+[0 0.1*dt 0.1*dt 0.3*dt 0.3*dt] tfinal; ...
   %                  [-0.01  -0.01    0.9    0.9   0.99    0.99]*lb; ...
   %                      1      0      1      0      1       0]];
   lb = ylim(2)-ylim(1)*0.9;
   %   NCD_LB = [NCD_LB, ...
   ncdStruct.CnstrLB = [ncdStruct.CnstrLB,...
         [                                out*ones(1,6); ...
            tstart+[0 0.23*dt 0.23*dt 0.3*dt 0.3*dt] tfinal; ...
            [-0.01  -0.01    0.9    0.9   0.99    0.99]; ...
            1      0      1      0      1       0]];
else
   
   % Do some error checking on the lower bounds
   % Four rows
   % Outputs are integers
   % Even number of columns for every output
   % x(i+1)>x(i) for i = 1:size(x,2)-1
   % x(i+1)=x(i) for i = 2:2:size(x,2)-2
   
   error_str = '';
   tmp = ncdStruct.CnstrLB;
   
   if size(tmp,1)~=4,
      % Constraint bound must have four rows
      error_str = 'FILLAXES: ncdStruct.CnstrLB must have four rows. ';
   elseif any( tmp(1,:)~=round(tmp(1,:)) ),
      error_str = 'FILLAXES: ncdStruct.CnstrLB(1,:) must be integers. ';
   else
      indx = find(tmp(1,:)==out);
      x = tmp(2,indx);
      lx = length(indx);
      
      if x(1) < tstart,
         tstart = x(1);
         set_param(sys,'Start time',num2str(tstart));
      end
      if x(end) > tfinal,
         tfinal = x(end);
         set_param(sys,'Stop time',num2str(tfinal));
      end
      
      % Error checks
      if rem(lx,2),
         error_str = 'FILLAXES: ncdStruct.CnstrLB must have an even number of columns.';
      elseif any( x(2:lx) < x(1:lx-1) ),
         error_str = 'FILLAXES: ncdStruct.CnstrLB(2,:) must be monotonic. ';
      elseif any( x(2:2:lx-2) ~= x(3:2:lx-1) ),
         error_str = 'FILLAXES: Segments in ncdStruct.CnstrLB should begin where another ends. ';
      end
   end
   if ~isempty(error_str),
      error(error_str);
   end
end

% If no upper bounds defined, use default
DoDefault = 0;
DoDefault = 0;
if isempty(ncdStruct.CnstrUB),
   DoDefault = 1;
elseif isempty(find(ncdStruct.CnstrUB(1,:)==out)),
   DoDefault = 1;
end

if DoDefault,
   dt = tfinal-tstart;
   ylim = get(gca, 'Ylim');
   %   ub = ylim(2) - (ylim(2)-ylim(1))*.1;
   %   NCD_UB = [NCD_UB, ...
   %             [out*ones(1,4); ...
   %              tstart+[0 0.5*dt 0.5*dt] tfinal; ...
   %              ub ub ub ub; ...
   %              1  0  1  0]];
   %   ub = (ylim(2)-ylim(1))*0.9;
   %   NCD_UB = [NCD_UB, ...
   %             [                  out*ones(1,4); ...
   %              tstart+[0 0.3*dt 0.3*dt] tfinal; ...
   %                     [1.2   1.2   1.01    1.01]*ub; ...
   %                      1      0      1       0]];
   %   NCD_UB = [NCD_UB, ...
   ncdStruct.CnstrUB = [ncdStruct.CnstrUB, ...
         [                  out*ones(1,6); ...
            tstart+[0 0.19*dt 0.19*dt 0.3*dt 0.3*dt] tfinal; ...
            [0.01 0.01 1.1   1.1   1.01    1.01]; ...
            1   0   1      0      1       0]];
else
   
   % Do some error checking on the upper bounds
   % Four rows
   % Outputs are integers
   % Even number of columns for every output
   % x(i+1)>x(i) for i = 1:size(x,2)-1
   % x(i+1)=x(i) for i = 2:2:size(x,2)-2
   
   error_str = '';
   tmp = ncdStruct.CnstrUB;
   
   if size(tmp,1)~=4,
      % Constraint bound must have four rows
      error_str = 'FILLAXES: ncdStruct.CnstrUB must have four rows. ';
   elseif any( tmp(1,:)~=round(tmp(1,:)) ),
      error_str = 'FILLAXES: ncdStruct.CnstrUB(1,:) must be integers. ';
   else
      indx = find(tmp(1,:)==out);
      x = tmp(2,indx);
      lx = length(indx);
      
      if x(1) < tstart,
         tstart = x(1);
         set_param(sys,'Start time',num2str(tstart));
      end
      if x(end) > tfinal,
         tfinal = x(end);
         set_param(sys,'Stop time',num2str(tfinal));
      end
      
      % Error checks
      if rem(lx,2),
         error_str = 'FILLAXES: ncdStruct.CnstrUB must have an even number of columns.';
      elseif any( x(2:lx) < x(1:lx-1) ),
         error_str = 'FILLAXES: ncdStruct.CnstrUB(2,:) must be monotonic. ';
      elseif any( x(2:2:lx-2) ~= x(3:2:lx-1) ),
         error_str = 'FILLAXES: Segments in ncdStruct.CnstrUB should begin where another ends. ';
      end
   end
   if ~isempty(error_str),
      error(error_str);
   end
end

% Initialize RngLmts and set axes limits

xlim = [tstart tfinal];

Lindx = find(ncdStruct.CnstrLB(1,:)==out);
Uindx = find(ncdStruct.CnstrUB(1,:)==out);
ylim(1,1) = min([ncdStruct.CnstrLB(3,Lindx) ncdStruct.CnstrUB(3,Uindx)]);
ylim(1,2) = max([ncdStruct.CnstrLB(3,Lindx) ncdStruct.CnstrUB(3,Uindx)]);
yslack = max(0.1*(ylim(2)-ylim(1)),0.05);
ylim(1) = ylim(1)-yslack;
ylim(2) = ylim(2)+yslack;

if isempty(ncdStruct.RngLmts),
   ncdStruct.RngLmts = [out out; xlim; ylim];
elseif (size(ncdStruct.RngLmts,1) ~= 3) | ...
      (rem(size(ncdStruct.RngLmts,2),2) ~= 0),
   ncdStruct.RngLmts = [out out; xlim; ylim];
else
   indx = find(ncdStruct.RngLmts(1,:) == out);
   if isempty(indx),
      ncdStruct.RngLmts = [ncdStruct.RngLmts [out out; xlim; ylim]];
   elseif length(indx) ~= 2,
      ncdStruct.RngLmts(:,indx) = [];
      ncdStruct.RngLmts = [ncdStruct.RngLmts [out out; xlim; ylim]];
   else
      xlim(1) = min([ncdStruct.RngLmts(2,indx) xlim(1)]);
      xlim(2) = max([ncdStruct.RngLmts(2,indx) xlim(2)]);
      ylim(1) = min([ncdStruct.RngLmts(3,indx) ylim(1)]);
      ylim(2) = max([ncdStruct.RngLmts(3,indx) ylim(2)]);
      ncdStruct.RngLmts(2:3,indx) = [xlim; ylim];
   end
end;

set(axs,'XLim',xlim,'YLim',ylim);

% Draw constraint bounds for this output

indx = find(ncdStruct.CnstrLB(1,:)==out);
bound = ncdStruct.CnstrLB(2:4,indx);
[S_and_P(1),S_and_P(3)] = makesurf(fig,axs,bound,1);

indx = find(ncdStruct.CnstrUB(1,:)==out);
bound = ncdStruct.CnstrUB(2:4,indx);
[S_and_P(2),S_and_P(4)] = makesurf(fig,axs,bound,2);  

if (0==1)
   % Set axes limits
   ax = get(fig,'CurrentA');
   ylim = ylim + diff(ylim)*[-1 1]/10;
   %if size(RANGE_LIMIT,1)>=out+1,
   % Note: zeros denote y-axis should auto-scale
   %  if any(RANGE_LIMIT(out+1,:)~=[0 0]),
   %      ylim = RANGE_LIMIT(out+1,:);
   %   end
   %end
   yaxl = get(ax, 'YLim');
   ylim(1) = min(yaxl(1), ylim(1)); 
   ylim(2) = max(yaxl(2), ylim(2)); 
   set(ax,'XLim',RANGE_LIMIT(1,:),'YLim',ylim);
   %set(ax,'XLim',RANGE_LIMIT(1,:));
   
   % Finally draw constraint bounds for this outpu
   %set(axs,'XLim',xlim,'YLim',ylim);
   set(axs,'YLim',ylim);
   index = find(NCD_LB(1,:)==out);
   bound = NCD_LB(2:4,index);
   NCD_LB(:,index) = [];
   [toosmall,bound] = dividecb(bound,tstart);
   [bound,toolarge] = dividecb(bound,tfinal);
   [S_and_P(1),S_and_P(3)] = makesurf(fig,axs,bound,1);
   bound = [toosmall bound toolarge];
   bound = [out*ones(1,size(bound,2)); bound];
   NCD_LB = [NCD_LB bound];
   
   index = find(NCD_UB(1,:)==out);
   bound = NCD_UB(2:4,index);
   NCD_UB(:,index) = [];
   [toosmall,bound] = dividecb(bound,tstart);
   [bound,toolarge] = dividecb(bound,tfinal);
   [S_and_P(2),S_and_P(4)] = makesurf(fig,axs,bound,2);  
   bound = [toosmall bound toolarge];
   bound = [out*ones(1,size(bound,2)); bound];
   NCD_UB = [NCD_UB bound];
   
end;
% end fillaxes
