function [sys, x0] = snefcon(t, x, u, flag, pnefmat);  
%SNEFCON SIMULINK s-function calling NEFCON.
%
% See also NEFCON.  
   
% Author: Andreas Nuernberger, University of Magdeburg, 02/98; andreas.nuernberger@cs.uni-magdeburg.de

% Globals:  
global NEFCON_STAT  
global LASTTIME SYSOLD UOLD 
global NEFCON_NUMIN  
global NEFCON_STOP  
global NEFCON_FIS
global NEFCON_SYSNAME
global NEFCON_NOISE

OUT = 4;          % output-, fuzzy_error, stop-signal  
IN = NEFCON_NUMIN;  
  
if flag == 0
   if isempty(NEFCON_SYSNAME)
      % system not initialized - load settings 
      if nargin < 5
         pnefmat = [];
      end
      if isempty(pnefmat)
         NEFCON_SYSNAME = get_param(0,'CurrentSystem'); 
         if ~isempty(find(NEFCON_SYSNAME=='/'))
            NEFCON_SYSNAME = NEFCON_SYSNAME(1:find(NEFCON_SYSNAME=='/')-1);
         end
         %      if ~isempty(find(fn=='\'))
         %         NEFCON_SYSNAME = NEFCON_SYSNAME(1:find(fn=='\')-1));
         %      end
         path = [pwd '/']; 
         fn = [ NEFCON_SYSNAME, '.nef' ]; 
      else 
         fn = pnefmat;
         if isempty(find(fn=='/'))
            path = [pwd '/']; 
         else
            path = '';
         end
      end
      if exist([path fn]) == 0  
         error(['SNEFCON: Error loading initial NEFCON settings ' [path fn]]);
         NEFCON_STOP = 1
      else  
         % set globals  
         eval(['load ' [path fn] ' -mat; nefmat(nefsmat);']);     % loads matrix smat  
      end
   end
   if NEFCON_STAT == 0 | isempty(NEFCON_STAT)
      IN = NEFCON_NUMIN;  
      SYSOLD = nefcon(zeros(1,IN+1));  % init_nefcon  
      %SYSOLD = [0,0]
      LASTTIME = -1;  
   end  
   sys = [0, 0, OUT, IN, 0, 0];  
   
   %    x0 =[0; 0]                      % ???  
   
elseif flag == 3  
   % prevent duplicate calls during one timestep
   %    if LASTTIME ~= t | UOLD ~= u | SYSOLD == [] 
   if LASTTIME ~= t | UOLD ~= u
      %        if LASTTIME == t   
      %            out = evalfis(u', NEFCON_FIS)  
      %            SYSOLD=[out; 0];   
      %        else
      SYSOLD = nefcon([t u']); 
      LASTTIME = t; 
      %        end
      UOLD = u;
   end  
   sys = [SYSOLD; NEFCON_STOP; NEFCON_NOISE]; 
else  
   
   sys = [];  
   
end  

