function [sys, x0] = snefsign(t, x, u, flag); 
%SNEFCON SIMULINK s-function Nefcon Bounded Signal Generator 
%        This funtion interpolates bounds an signal at timestep t.
%
% See also NEFCON and NEFOPTF.  
   
%	Author(s): Andreas Nuernberger, 08/1996 
 
% NCD Globals: 
%global NCD_LB NCD_UB NEFCON_SIGNAL ncdStruct
global NEFCON_SIGNAL ncdStruct

%NEFCON globals
global NEFCON_BOUNDS

NCD_LB = ncdStruct.CnstrLB;
NCD_UB = ncdStruct.CnstrUB;

outp = 3; 
inp = 2; 
x0=[0];

if isempty(u) 
   u_out = [0,0];
else
   u_out = u';
end
 
if flag == 0 
    if isempty(NCD_LB) | isempty(NCD_UB)
        error('Bounds undefined.');
 
    end 
    if isempty(NEFCON_SIGNAL)
        error('Signal undefined.'); 
    end 
 
    sys = [0, 1, outp, inp, 0, 1]; 
    ub = NCD_UB(3,1); 
    lb = NCD_LB(3,1); 
    sign = NEFCON_SIGNAL(2,1); 

    NEFCON_BOUNDS = [sign, ub, lb, u_out];
 
elseif flag == 3 
 
    % find bounds 
 
    t_max = NCD_UB(2, size(NCD_UB,2)); 
    if t >= t_max 
        ub = NCD_UB(3,size(NCD_UB,2)); 
        lb = NCD_LB(3,size(NCD_LB,2)); 
        sign = NEFCON_SIGNAL(2, size(NEFCON_SIGNAL, 2)); 
    elseif t <= NCD_UB(2,1) 
        ub = NCD_UB(3,1); 
        lb = NCD_LB(3,1); 
        sign = NEFCON_SIGNAL(2,1); 
    else 
        % interpolate 
        row = 2; 
        while NCD_UB(2,row) < t 
            row = row + 2; 
        end; 
        ub = NCD_UB(3,row-1) ... 
                + (NCD_UB(3,row)-NCD_UB(3,row-1))/(NCD_UB(2,row)-NCD_UB(2,row-1)) ... 
                * (t - NCD_UB(2,row-1)); 
        row = 2; 
        while NCD_LB(2,row) < t 
            row = row + 2; 
        end; 
        lb = NCD_LB(3,row-1) ... 
                + (NCD_LB(3,row)-NCD_LB(3,row-1))/(NCD_LB(2,row)-NCD_LB(2,row-1)) ... 
                * (t - NCD_LB(2,row-1)); 
        row = 2; 
        while NEFCON_SIGNAL(1,row) < t 
            row = row + 1; 
        end; 
        sign = NEFCON_SIGNAL(2,row-1) ... 
                + (NEFCON_SIGNAL(2,row)-NEFCON_SIGNAL(2,row-1))/(NEFCON_SIGNAL(1,row)-NEFCON_SIGNAL(1,row-1)) ... 
                * (t - NEFCON_SIGNAL(1,row-1)); 
    end 
 
    sys = [sign, ub, lb]; 
    NEFCON_BOUNDS = [sys, u_out];
 
else 
 
    sys = []; 
 
end 
